package ovh.herisson.Clyde.Services.ScientificPublications;

import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.ScientificPublications.ResearchRepository;
import ovh.herisson.Clyde.Repositories.ScientificPublications.ResearcherRepository;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.ScientificPublications.*;
import ovh.herisson.Clyde.Tables.User;

import java.util.*;

@Service
@AllArgsConstructor
@SuppressWarnings("unchecked")
public class ResearchesService {

    private final ResearcherRepository researcherRepo;
    private final ResearchRepository articleRepo;


    // researches Part
    public Research getResearchById(long id) {
        return articleRepo.findById(id);
    }

    public Research getResearchByUrl(String url) {
        return articleRepo.findByPdfLocation(url);
    }


    public Iterable<Research> getResearchesByAuthor(long authorId){
        Researcher researcher = researcherRepo.findById(authorId);
        if (researcher == null) return null;

        return researcherRepo.findAllByAuthorId(researcher);
    }

    public Research saveResearch(Research research) {
        return articleRepo.save(research);
    }

    public void modifyResearchData(Research research, Map<String, Object> updates) {
        for (Map.Entry<String, Object> entry : updates.entrySet()){
            switch (entry.getKey()){
                case "title":
                    research.setTitle((String) entry.getValue());
                    break;
                case "paperType":
                    research.setPaperType((PaperType) entry.getValue());
                    break;
                case "language":
                    research.setLanguage((String) entry.getValue());
                    break;
                case "domain":
                    research.setDomain((String) entry.getValue());
                    break;
                case "summary":
                    research.setSummary((String) entry.getValue());
                    break;
                case "access":
                    research.setAccess(Access.valueOf((String) entry.getValue()));
                    break;
                case "coAuthors":
                    Set<Researcher> set = new HashSet<>();

                    for (int id : (List<Integer>) entry.getValue()) {

                        Researcher r = researcherRepo.findById(id);
                        if (r != null){
                            set.add(r);
                        }
                    }
                    research.setCoAuthors(set);
                    break;
            }
        }
        articleRepo.save(research);
    }

    public void deleteResearch(Research research) {
        articleRepo.delete(research);
    }


    // Researchers Part
    public Researcher getResearcherByUser(User user){
        return researcherRepo.findByUser(user);
    }

    public Iterable<Research> getAllResearches() {
        return articleRepo.findAll();
    }

    public Researcher saveResearcher(Researcher researcher) {

        if (researcherRepo.findByUser(researcher.getUser()) != null) return null;
       return researcherRepo.save(researcher);
    }

    public Iterable<Researcher> getAllResearchers() {
        return researcherRepo.findAll();
    }

    public Researcher getResearcherById(long id) {
        return  researcherRepo.findById(id);
    }

    public void deleteResearcher(Researcher researcher) {
       researcherRepo.delete(researcher);
    }

    public void modifyResearcherData(Researcher researcher, Map<String, Object> updates) {

        for (Map.Entry<String, Object> entry : updates.entrySet()){
            switch (entry.getKey()){
                case "orcidId":
                    if (entry.getValue() != null)
                        researcher.setOrcidId((String) entry.getValue());
                    break;
                case "domain":
                    if (entry.getValue() != null)
                        researcher.setDomain((String) entry.getValue());
                    break;
                case "site":
                    if (entry.getValue() != null)
                        researcher.setSite((String) entry.getValue());
                    break;
           }
        }
        researcherRepo.save(researcher);
    }


    //Co Author part
    public boolean saveCoAuthors(Iterable<Long> researchersId, Research research) {

        if (researchersId == null) return false;

        ArrayList<Researcher> toAdd = new ArrayList<>();
        for (long researcherId : researchersId){
            Researcher researcher= researcherRepo.findById(researcherId);
            if (researcher== null){
                return false;
            }
            if (!toAdd.contains(researcher))
            {
                toAdd.add(researcher);
            }
        }
        research.getCoAuthors().addAll(toAdd);
        articleRepo.save(research);
        return true;
    }

    public boolean deleteCoAuthor(Research research,Researcher coAuthor) {

        if (!research.getCoAuthors().contains(coAuthor))
            return false;

        research.getCoAuthors().remove(coAuthor);
        articleRepo.save(research);
        return true;
    }

    // Other stuff

    public Research addView(Research research) {
        research.setViews(research.getViews()+1);
        return articleRepo.save(research);
    }

    public boolean hasNoAccessTo(Research research, User user){
        if (research.getAccess() == Access.OpenSource) return false; // if the access is open source even non-users can see it
        if (user == null) return true; // else you need at least to be a user

        if (user.getRole() == Role.Admin) return false;


        Researcher researcher = getResearcherByUser(user);
        if (researcher !=null ){
            if (research.getAuthor().getId() == researcher.getId())
                return false;

            for (Researcher coAuthor: research.getCoAuthors()){
                if (coAuthor.getId() == researcher.getId())
                    return false;
            }
        }

        if (research.getAccess() == Access.Restricted && (
                user.getRole() == Role.Secretary ||
                user.getRole() == Role.Teacher || user.getRole() == Role.InscriptionService))
            return false; // if the access is restricted only the staff member (above) can access the research

        return true;
        // if the researcher is the author or one of the co-authors of the research will return false
    }
}
